@@ -103,10 +103,10 @@ gst_kate_parse_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_kate_parse_src_factory));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_kate_parse_sink_factory));
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_kate_parse_src_factory);
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_kate_parse_sink_factory);
   gst_element_class_set_details_simple (element_class, "Kate stream parser",
       "Codec/Parser/Subtitle",
       "parse raw kate streams",
