@@ -3,6 +3,7 @@
  * Copyright 2005 Thomas Vander Stichele <thomas@apestaart.org>
  * Copyright 2005 Ronald S. Bultje <rbultje@ronald.bitfreak.net>
  * Copyright 2008 Vincent Penquerc'h <ogg.k.ogg.k@googlemail.com>
+ * Copyright (C) <2009> Young-Ho Cha <ganadist@gmail.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -121,9 +122,61 @@ enum
   ARG_DEFAULT_BACKGROUND_RED,
   ARG_DEFAULT_BACKGROUND_GREEN,
   ARG_DEFAULT_BACKGROUND_BLUE,
-  ARG_DEFAULT_BACKGROUND_ALPHA
+  ARG_DEFAULT_BACKGROUND_ALPHA,
+  ARG_SILENT
 };
 
+/* RGB -> YUV blitting routines taken from textoverlay,
+   original code from Young-Ho Cha <ganadist@gmail.com> */
+
+#define COMP_Y(ret, r, g, b) \
+{ \
+   ret = (int) (((19595 * r) >> 16) + ((38470 * g) >> 16) + ((7471 * b) >> 16)); \
+   ret = CLAMP (ret, 0, 255); \
+}
+
+#define COMP_U(ret, r, g, b) \
+{ \
+   ret = (int) (-((11059 * r) >> 16) - ((21709 * g) >> 16) + ((32768 * b) >> 16) + 128); \
+   ret = CLAMP (ret, 0, 255); \
+}
+
+#define COMP_V(ret, r, g, b) \
+{ \
+   ret = (int) (((32768 * r) >> 16) - ((27439 * g) >> 16) - ((5329 * b) >> 16) + 128); \
+   ret = CLAMP (ret, 0, 255); \
+}
+
+#define BLEND(ret, alpha, v0, v1) \
+{ \
+	ret = (v0 * alpha + v1 * (255 - alpha)) / 255; \
+}
+
+#define OVER(ret, alphaA, Ca, alphaB, Cb, alphaNew)	\
+{ \
+    gint _tmp; \
+    _tmp = (Ca * alphaA + Cb * alphaB * (255 - alphaA) / 255) / alphaNew; \
+    ret = CLAMP (_tmp, 0, 255); \
+}
+
+#if G_BYTE_ORDER == G_LITTLE_ENDIAN
+# define TIGER_ARGB_A 3
+# define TIGER_ARGB_R 2
+# define TIGER_ARGB_G 1
+# define TIGER_ARGB_B 0
+#else
+# define TIGER_ARGB_A 0
+# define TIGER_ARGB_R 1
+# define TIGER_ARGB_G 2
+# define TIGER_ARGB_B 3
+#endif
+
+#define TIGER_UNPREMULTIPLY(a,r,g,b) G_STMT_START { \
+  b = (a > 0) ? MIN ((b * 255 + a / 2) / a, 255) : 0; \
+  g = (a > 0) ? MIN ((g * 255 + a / 2) / a, 255) : 0; \
+  r = (a > 0) ? MIN ((r * 255 + a / 2) / a, 255) : 0; \
+} G_STMT_END
+
 static GstStaticPadTemplate kate_sink_factory =
     GST_STATIC_PAD_TEMPLATE ("subtitle_sink",
     GST_PAD_SINK,
@@ -133,12 +186,12 @@ static GstStaticPadTemplate kate_sink_factory =
 
 #if G_BYTE_ORDER == G_LITTLE_ENDIAN
 #define TIGER_VIDEO_CAPS \
-    GST_VIDEO_CAPS_xRGB ", endianness = (int)1234; " \
-    GST_VIDEO_CAPS_BGRx ", endianness = (int)4321"
+    GST_VIDEO_CAPS_xRGB ";" GST_VIDEO_CAPS_BGRx ";" \
+    GST_VIDEO_CAPS_YUV ("{AYUV, I420, YV12, UYVY, NV12, NV21}")
 #else
 #define TIGER_VIDEO_CAPS \
-    GST_VIDEO_CAPS_BGRx ", endianness = (int)4321; " \
-    GST_VIDEO_CAPS_xRGB ", endianness = (int)1234"
+    GST_VIDEO_CAPS_BGRx ";" GST_VIDEO_CAPS_xRGB ";" \
+    GST_VIDEO_CAPS_YUV ("{AYUV, I420, YV12, UYVY, NV12, NV21}")
 #endif
 
 static GstStaticPadTemplate video_sink_factory =
@@ -194,12 +247,10 @@ gst_kate_tiger_base_init (gpointer gclass)
 
   GstElementClass *element_class = GST_ELEMENT_CLASS (gclass);
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&src_factory));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&kate_sink_factory));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&video_sink_factory));
+  gst_element_class_add_static_pad_template (element_class, &src_factory);
+  gst_element_class_add_static_pad_template (element_class, &kate_sink_factory);
+  gst_element_class_add_static_pad_template (element_class,
+      &video_sink_factory);
   gst_element_class_set_details_simple (element_class, "Kate stream renderer",
       "Mixer/Video/Overlay/Subtitle",
       "Decodes and renders Kate streams on top of a video",
@@ -294,6 +345,12 @@ gst_kate_tiger_class_init (GstKateTigerClass * klass)
           "Default background color (alpha component, between 0 and 255) to render text with",
           0, 255, 255, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
+  /* FIXME 0.11: rename to "visible" or "text-visible" or "render-text" */
+  g_object_class_install_property (gobject_class, ARG_SILENT,
+      g_param_spec_boolean ("silent", "silent",
+          "Whether to render the stream",
+          FALSE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
   gstelement_class->change_state =
       GST_DEBUG_FUNCPTR (gst_kate_tiger_change_state);
 }
@@ -309,6 +366,7 @@ gst_kate_tiger_init (GstKateTiger * tiger, GstKateTigerClass * gclass)
   GST_DEBUG_OBJECT (tiger, "gst_kate_tiger_init");
 
   tiger->mutex = g_mutex_new ();
+  tiger->cond = g_cond_new ();
 
   tiger->katesinkpad =
       gst_pad_new_from_static_template (&kate_sink_factory, "subtitle_sink");
@@ -352,6 +410,7 @@ gst_kate_tiger_init (GstKateTiger * tiger, GstKateTigerClass * gclass)
   tiger->default_background_g = 0;
   tiger->default_background_b = 0;
   tiger->default_background_a = 0;
+  tiger->silent = FALSE;
 
   tiger->video_width = 0;
   tiger->video_height = 0;
@@ -371,6 +430,12 @@ gst_kate_tiger_dispose (GObject * object)
     tiger->default_font_desc = NULL;
   }
 
+  g_free (tiger->render_buffer);
+  tiger->render_buffer = NULL;
+
+  g_cond_free (tiger->cond);
+  tiger->cond = NULL;
+
   g_mutex_free (tiger->mutex);
   tiger->mutex = NULL;
 
@@ -484,6 +549,9 @@ gst_kate_tiger_set_property (GObject * object, guint prop_id,
       tiger->default_background_a = g_value_get_int (value);
       gst_kate_tiger_update_default_background_color (tiger);
       break;
+    case ARG_SILENT:
+      tiger->silent = g_value_get_boolean (value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -538,6 +606,9 @@ gst_kate_tiger_get_property (GObject * object, guint prop_id,
     case ARG_DEFAULT_BACKGROUND_ALPHA:
       g_value_set_int (value, tiger->default_background_a);
       break;
+    case ARG_SILENT:
+      g_value_set_boolean (value, tiger->silent);
+      break;
     default:
       if (!gst_kate_util_decoder_base_get_property (&tiger->decoder, object,
               prop_id, value, pspec)) {
@@ -567,6 +638,12 @@ gst_kate_tiger_kate_chain (GstPad * pad, GstBuffer * buf)
   GST_LOG_OBJECT (tiger, "Got kate buffer, caps %" GST_PTR_FORMAT,
       GST_BUFFER_CAPS (buf));
 
+  /* Now that we have the lock, check if we're flushing */
+  if (tiger->decoder.kate_flushing) {
+    GST_DEBUG_OBJECT (tiger, "Flushing, disregarding buffer");
+    goto done;
+  }
+
   /* Unfortunately, it can happen that the start of the stream is not sent,
      for instance if there's a stream selector upstream, which is switched
      from another Kate stream. If this happens, then we can fallback on the
@@ -631,6 +708,35 @@ gst_kate_tiger_kate_chain (GstPad * pad, GstBuffer * buf)
     gst_object_unref (tagpad);
   }
 
+  /* we want to avoid shooting ahead of the video stream, or we will
+     get segment updates which will place us ahead of it, and we won't
+     be able to convert a video timestamp back into a kate timestamp */
+  if (G_LIKELY (GST_BUFFER_TIMESTAMP_IS_VALID (buf))) {
+    while (1) {
+      gint64 kate_time, video_time;
+      kate_time =
+          gst_segment_to_running_time (&tiger->decoder.kate_segment,
+          GST_FORMAT_TIME, GST_BUFFER_TIMESTAMP (buf));
+      video_time =
+          gst_segment_to_running_time (&tiger->video_segment, GST_FORMAT_TIME,
+          tiger->video_segment.last_stop);
+      GST_DEBUG_OBJECT (tiger, "Kate time %.2f, video time %.2f (kts %ld)",
+          kate_time / (float) GST_SECOND, video_time / (float) GST_SECOND,
+          (long) GST_BUFFER_TIMESTAMP (buf));
+      if (kate_time <= video_time) {
+        break;
+      }
+      GST_LOG_OBJECT (tiger, "Waiting to return from chain function");
+      g_cond_wait (tiger->cond, tiger->mutex);
+      if (tiger->decoder.kate_flushing) {
+        GST_DEBUG_OBJECT (tiger, "Flushing while waiting");
+        break;
+      }
+      GST_LOG_OBJECT (tiger, "Woken up, checking time again");
+    }
+  }
+
+done:
   GST_KATE_TIGER_MUTEX_UNLOCK (tiger);
 
   gst_object_unref (tiger);
@@ -645,6 +751,7 @@ gst_kate_tiger_video_set_caps (GstPad * pad, GstCaps * caps)
   GstKateTiger *tiger = GST_KATE_TIGER (gst_pad_get_parent (pad));
   GstVideoFormat format;
   gint w, h;
+  gboolean ret;
 
   GST_KATE_TIGER_MUTEX_LOCK (tiger);
 
@@ -654,23 +761,424 @@ gst_kate_tiger_video_set_caps (GstPad * pad, GstCaps * caps)
   tiger->swap_rgb = FALSE;
 
   if (gst_video_format_parse_caps (caps, &format, &w, &h)) {
+    tiger->video_format = format;
     tiger->video_width = w;
     tiger->video_height = h;
   }
 
   GST_KATE_TIGER_MUTEX_UNLOCK (tiger);
 
-  gst_pad_set_caps (tiger->srcpad, caps);
+  ret = gst_pad_set_caps (tiger->srcpad, caps);
 
   gst_object_unref (tiger);
-  return TRUE;
+  return ret;
 }
 
 static gdouble
 gst_kate_tiger_get_time (GstKateTiger * tiger)
 {
-  return gst_segment_to_running_time (&tiger->video_segment, GST_FORMAT_TIME,
-      tiger->video_segment.last_stop) / (gdouble) GST_SECOND;
+  gint64 rt =
+      gst_segment_to_running_time (&tiger->video_segment, GST_FORMAT_TIME,
+      tiger->video_segment.last_stop);
+  gint64 pos =
+      gst_segment_to_position (&tiger->decoder.kate_segment, GST_FORMAT_TIME,
+      rt);
+  return pos / (gdouble) GST_SECOND;
+}
+
+static inline void
+gst_kate_tiger_blit_1 (GstKateTiger * tiger, guchar * dest, gint xpos,
+    gint ypos, const guint8 * image, gint image_width, gint image_height,
+    guint dest_stride)
+{
+  gint i, j = 0;
+  gint x, y;
+  guchar r, g, b, a;
+  const guint8 *pimage;
+  guchar *py;
+  gint width = image_width;
+  gint height = image_height;
+
+  if (xpos < 0) {
+    xpos = 0;
+  }
+
+  if (xpos + width > tiger->video_width) {
+    width = tiger->video_width - xpos;
+  }
+
+  if (ypos + height > tiger->video_height) {
+    height = tiger->video_height - ypos;
+  }
+
+  dest += (ypos / 1) * dest_stride;
+
+  for (i = 0; i < height; i++) {
+    pimage = image + 4 * (i * image_width);
+    py = dest + i * dest_stride + xpos;
+    for (j = 0; j < width; j++) {
+      b = pimage[TIGER_ARGB_B];
+      g = pimage[TIGER_ARGB_G];
+      r = pimage[TIGER_ARGB_R];
+      a = pimage[TIGER_ARGB_A];
+      TIGER_UNPREMULTIPLY (a, r, g, b);
+
+      pimage += 4;
+      if (a == 0) {
+        py++;
+        continue;
+      }
+      COMP_Y (y, r, g, b);
+      x = *py;
+      BLEND (*py++, a, y, x);
+    }
+  }
+}
+
+static inline void
+gst_kate_tiger_blit_sub2x2cbcr (GstKateTiger * tiger,
+    guchar * destcb, guchar * destcr, gint xpos, gint ypos,
+    const guint8 * image, gint image_width, gint image_height,
+    guint destcb_stride, guint destcr_stride, guint pix_stride)
+{
+  gint i, j;
+  gint x, cb, cr;
+  gushort r, g, b, a;
+  gushort r1, g1, b1, a1;
+  const guint8 *pimage1, *pimage2;
+  guchar *pcb, *pcr;
+  gint width = image_width - 2;
+  gint height = image_height - 2;
+
+  xpos *= pix_stride;
+
+  if (xpos < 0) {
+    xpos = 0;
+  }
+
+  if (xpos + width > tiger->video_width) {
+    width = tiger->video_width - xpos;
+  }
+
+  if (ypos + height > tiger->video_height) {
+    height = tiger->video_height - ypos;
+  }
+
+  destcb += (ypos / 2) * destcb_stride;
+  destcr += (ypos / 2) * destcr_stride;
+
+  for (i = 0; i < height; i += 2) {
+    pimage1 = image + 4 * (i * image_width);
+    pimage2 = pimage1 + 4 * image_width;
+    pcb = destcb + (i / 2) * destcb_stride + xpos / 2;
+    pcr = destcr + (i / 2) * destcr_stride + xpos / 2;
+    for (j = 0; j < width; j += 2) {
+      b = pimage1[TIGER_ARGB_B];
+      g = pimage1[TIGER_ARGB_G];
+      r = pimage1[TIGER_ARGB_R];
+      a = pimage1[TIGER_ARGB_A];
+      TIGER_UNPREMULTIPLY (a, r, g, b);
+      pimage1 += 4;
+
+      b1 = pimage1[TIGER_ARGB_B];
+      g1 = pimage1[TIGER_ARGB_G];
+      r1 = pimage1[TIGER_ARGB_R];
+      a1 = pimage1[TIGER_ARGB_A];
+      TIGER_UNPREMULTIPLY (a1, r1, g1, b1);
+      b += b1;
+      g += g1;
+      r += r1;
+      a += a1;
+      pimage1 += 4;
+
+      b1 = pimage2[TIGER_ARGB_B];
+      g1 = pimage2[TIGER_ARGB_G];
+      r1 = pimage2[TIGER_ARGB_R];
+      a1 = pimage2[TIGER_ARGB_A];
+      TIGER_UNPREMULTIPLY (a1, r1, g1, b1);
+      b += b1;
+      g += g1;
+      r += r1;
+      a += a1;
+      pimage2 += 4;
+
+      /* + 2 for rounding */
+      b1 = pimage2[TIGER_ARGB_B];
+      g1 = pimage2[TIGER_ARGB_G];
+      r1 = pimage2[TIGER_ARGB_R];
+      a1 = pimage2[TIGER_ARGB_A];
+      TIGER_UNPREMULTIPLY (a1, r1, g1, b1);
+      b += b1 + 2;
+      g += g1 + 2;
+      r += r1 + 2;
+      a += a1 + 2;
+      pimage2 += 4;
+
+      b /= 4;
+      g /= 4;
+      r /= 4;
+      a /= 4;
+
+      if (a == 0) {
+        pcb += pix_stride;
+        pcr += pix_stride;
+        continue;
+      }
+      COMP_U (cb, r, g, b);
+      COMP_V (cr, r, g, b);
+
+      x = *pcb;
+      BLEND (*pcb, a, cb, x);
+      x = *pcr;
+      BLEND (*pcr, a, cr, x);
+
+      pcb += pix_stride;
+      pcr += pix_stride;
+    }
+  }
+}
+
+/* FIXME:
+ *  - use proper strides and offset for I420
+ */
+
+static inline void
+gst_kate_tiger_blit_NV12_NV21 (GstKateTiger * tiger,
+    guint8 * yuv_pixels, gint xpos, gint ypos, const guint8 * image,
+    gint image_width, gint image_height)
+{
+  int y_stride, uv_stride;
+  int u_offset, v_offset;
+  int h, w;
+
+  /* because U/V is 2x2 subsampled, we need to round, either up or down,
+   * to a boundary of integer number of U/V pixels:
+   */
+  xpos = GST_ROUND_UP_2 (xpos);
+  ypos = GST_ROUND_UP_2 (ypos);
+
+  w = tiger->video_width;
+  h = tiger->video_height;
+
+  y_stride = gst_video_format_get_row_stride (tiger->video_format, 0, w);
+  uv_stride = gst_video_format_get_row_stride (tiger->video_format, 1, w);
+  u_offset =
+      gst_video_format_get_component_offset (tiger->video_format, 1, w, h);
+  v_offset =
+      gst_video_format_get_component_offset (tiger->video_format, 2, w, h);
+
+  gst_kate_tiger_blit_1 (tiger, yuv_pixels, xpos, ypos, image, image_width,
+      image_height, y_stride);
+  gst_kate_tiger_blit_sub2x2cbcr (tiger, yuv_pixels + u_offset,
+      yuv_pixels + v_offset, xpos, ypos, image, image_width, image_height,
+      uv_stride, uv_stride, 2);
+}
+
+static inline void
+gst_kate_tiger_blit_I420_YV12 (GstKateTiger * tiger,
+    guint8 * yuv_pixels, gint xpos, gint ypos, const guint8 * image,
+    gint image_width, gint image_height)
+{
+  int y_stride, u_stride, v_stride;
+  int u_offset, v_offset;
+  int h, w;
+
+  /* because U/V is 2x2 subsampled, we need to round, either up or down,
+   * to a boundary of integer number of U/V pixels:
+   */
+  xpos = GST_ROUND_UP_2 (xpos);
+  ypos = GST_ROUND_UP_2 (ypos);
+
+  w = tiger->video_width;
+  h = tiger->video_height;
+
+  y_stride = gst_video_format_get_row_stride (tiger->video_format, 0, w);
+  u_stride = gst_video_format_get_row_stride (tiger->video_format, 1, w);
+  v_stride = gst_video_format_get_row_stride (tiger->video_format, 2, w);
+  u_offset =
+      gst_video_format_get_component_offset (tiger->video_format, 1, w, h);
+  v_offset =
+      gst_video_format_get_component_offset (tiger->video_format, 2, w, h);
+
+  gst_kate_tiger_blit_1 (tiger, yuv_pixels, xpos, ypos, image, image_width,
+      image_height, y_stride);
+  gst_kate_tiger_blit_sub2x2cbcr (tiger, yuv_pixels + u_offset,
+      yuv_pixels + v_offset, xpos, ypos, image, image_width, image_height,
+      u_stride, v_stride, 1);
+}
+
+static inline void
+gst_kate_tiger_blit_UYVY (GstKateTiger * tiger,
+    guint8 * yuv_pixels, gint xpos, gint ypos, const guint8 * image,
+    gint image_width, gint image_height)
+{
+  int a0, r0, g0, b0;
+  int a1, r1, g1, b1;
+  int y0, y1, u, v;
+  int i, j;
+  int h, w;
+  const guint8 *pimage;
+  guchar *dest;
+
+  /* because U/V is 2x horizontally subsampled, we need to round to a
+   * boundary of integer number of U/V pixels in x dimension:
+   */
+  xpos = GST_ROUND_UP_2 (xpos);
+
+  w = image_width - 2;
+  h = image_height - 2;
+
+  if (xpos < 0) {
+    xpos = 0;
+  }
+
+  if (xpos + w > tiger->video_width) {
+    w = tiger->video_width - xpos;
+  }
+
+  if (ypos + h > tiger->video_height) {
+    h = tiger->video_height - ypos;
+  }
+
+  for (i = 0; i < h; i++) {
+    pimage = image + i * image_width * 4;
+    dest = yuv_pixels + (i + ypos) * tiger->video_width * 2 + xpos * 2;
+    for (j = 0; j < w; j += 2) {
+      b0 = pimage[TIGER_ARGB_B];
+      g0 = pimage[TIGER_ARGB_G];
+      r0 = pimage[TIGER_ARGB_R];
+      a0 = pimage[TIGER_ARGB_A];
+      TIGER_UNPREMULTIPLY (a0, r0, g0, b0);
+      pimage += 4;
+
+      b1 = pimage[TIGER_ARGB_B];
+      g1 = pimage[TIGER_ARGB_G];
+      r1 = pimage[TIGER_ARGB_R];
+      a1 = pimage[TIGER_ARGB_A];
+      TIGER_UNPREMULTIPLY (a1, r1, g1, b1);
+      pimage += 4;
+
+      a0 += a1 + 2;
+      a0 /= 2;
+      if (a0 == 0) {
+        dest += 4;
+        continue;
+      }
+
+      COMP_Y (y0, r0, g0, b0);
+      COMP_Y (y1, r1, g1, b1);
+
+      b0 += b1 + 2;
+      g0 += g1 + 2;
+      r0 += r1 + 2;
+
+      b0 /= 2;
+      g0 /= 2;
+      r0 /= 2;
+
+      COMP_U (u, r0, g0, b0);
+      COMP_V (v, r0, g0, b0);
+
+      BLEND (*dest, a0, u, *dest);
+      dest++;
+      BLEND (*dest, a0, y0, *dest);
+      dest++;
+      BLEND (*dest, a0, v, *dest);
+      dest++;
+      BLEND (*dest, a0, y1, *dest);
+      dest++;
+    }
+  }
+}
+
+static inline void
+gst_kate_tiger_blit_AYUV (GstKateTiger * tiger,
+    guint8 * rgb_pixels, gint xpos, gint ypos, const guint8 * image,
+    gint image_width, gint image_height)
+{
+  int a, r, g, b, a1;
+  int y, u, v;
+  int i, j;
+  int h, w;
+  const guint8 *pimage;
+  guchar *dest;
+
+  w = image_width;
+  h = image_height;
+
+  if (xpos < 0) {
+    xpos = 0;
+  }
+
+  if (xpos + w > tiger->video_width) {
+    w = tiger->video_width - xpos;
+  }
+
+  if (ypos + h > tiger->video_height) {
+    h = tiger->video_height - ypos;
+  }
+
+  for (i = 0; i < h; i++) {
+    pimage = image + i * image_width * 4;
+    dest = rgb_pixels + (i + ypos) * 4 * tiger->video_width + xpos * 4;
+    for (j = 0; j < w; j++) {
+      a = pimage[TIGER_ARGB_A];
+      b = pimage[TIGER_ARGB_B];
+      g = pimage[TIGER_ARGB_G];
+      r = pimage[TIGER_ARGB_R];
+
+      TIGER_UNPREMULTIPLY (a, r, g, b);
+
+      // convert background to yuv
+      COMP_Y (y, r, g, b);
+      COMP_U (u, r, g, b);
+      COMP_V (v, r, g, b);
+
+      // preform text "OVER" background alpha compositing
+      a1 = a + (dest[0] * (255 - a)) / 255 + 1; // add 1 to prevent divide by 0
+      OVER (dest[1], a, y, dest[0], dest[1], a1);
+      OVER (dest[2], a, u, dest[0], dest[2], a1);
+      OVER (dest[3], a, v, dest[0], dest[3], a1);
+      dest[0] = a1 - 1;         // remove the temporary 1 we added
+
+      pimage += 4;
+      dest += 4;
+    }
+  }
+}
+
+static void
+gst_kate_tiger_blend_yuv (GstKateTiger * tiger, GstBuffer * video_frame,
+    const guint8 * image, gint image_width, gint image_height)
+{
+  gint xpos = 0, ypos = 0;
+
+  switch (tiger->video_format) {
+    case GST_VIDEO_FORMAT_I420:
+    case GST_VIDEO_FORMAT_YV12:
+      gst_kate_tiger_blit_I420_YV12 (tiger,
+          GST_BUFFER_DATA (video_frame), xpos, ypos, image, image_width,
+          image_height);
+      break;
+    case GST_VIDEO_FORMAT_NV12:
+    case GST_VIDEO_FORMAT_NV21:
+      gst_kate_tiger_blit_NV12_NV21 (tiger,
+          GST_BUFFER_DATA (video_frame), xpos, ypos, image, image_width,
+          image_height);
+      break;
+    case GST_VIDEO_FORMAT_UYVY:
+      gst_kate_tiger_blit_UYVY (tiger,
+          GST_BUFFER_DATA (video_frame), xpos, ypos, image, image_width,
+          image_height);
+      break;
+    case GST_VIDEO_FORMAT_AYUV:
+      gst_kate_tiger_blit_AYUV (tiger,
+          GST_BUFFER_DATA (video_frame), xpos, ypos, image, image_width,
+          image_height);
+      break;
+    default:
+      g_assert_not_reached ();
+  }
 }
 
 static GstFlowReturn
@@ -680,6 +1188,7 @@ gst_kate_tiger_video_chain (GstPad * pad, GstBuffer * buf)
   GstFlowReturn rflow = GST_FLOW_OK;
   unsigned char *ptr;
   int ret;
+  kate_float t;
 
   GST_KATE_TIGER_MUTEX_LOCK (tiger);
 
@@ -695,47 +1204,79 @@ gst_kate_tiger_video_chain (GstPad * pad, GstBuffer * buf)
   if (G_LIKELY (GST_BUFFER_TIMESTAMP_IS_VALID (buf))) {
     gst_segment_set_last_stop (&tiger->video_segment, GST_FORMAT_TIME,
         GST_BUFFER_TIMESTAMP (buf));
+    g_cond_broadcast (tiger->cond);
   }
 
-  /* draw on it */
+  /* Update first with a dummy buffer pointer we cannot write to, but with the
+     right dimensions. If there is nothing to draw, we will not have to make
+     it writeable. */
+  ptr = GST_BUFFER_DATA (buf);
+  ret =
+      tiger_renderer_set_buffer (tiger->tr, ptr, tiger->video_width,
+      tiger->video_height, tiger->video_width * 4, tiger->swap_rgb);
+  if (G_UNLIKELY (ret < 0)) {
+    GST_WARNING_OBJECT (tiger,
+        "Tiger renderer failed to set buffer to video frame: %d", ret);
+    goto pass;
+  }
+
+  /* update the renderer at the time of the video frame */
+  t = gst_kate_tiger_get_time (tiger);
+  GST_LOG_OBJECT (tiger, "Video segment calc: last stop %ld, time %.3f",
+      (long) tiger->video_segment.last_stop, t);
+  ret = tiger_renderer_update (tiger->tr, t, 1);
+  if (G_UNLIKELY (ret < 0)) {
+    GST_WARNING_OBJECT (tiger, "Tiger renderer failed to update: %d", ret);
+    goto pass;
+  }
+
+  /* if there nothing to draw, we can just push the video buffer as is */
+  if (ret > 0 || tiger->silent)
+    goto pass;
+
+  /* there is something to draw, so first make the buffer writable */
   buf = gst_buffer_make_writable (buf);
   if (G_UNLIKELY (!buf)) {
     GST_WARNING_OBJECT (tiger, "Failed to make video buffer writable");
-  } else {
-    ptr = GST_BUFFER_DATA (buf);
-    if (!ptr) {
-      GST_WARNING_OBJECT (tiger,
-          "Failed to get a pointer to video buffer data");
-    } else {
-      ret =
-          tiger_renderer_set_buffer (tiger->tr, ptr, tiger->video_width,
-          tiger->video_height, tiger->video_width * 4, tiger->swap_rgb);
-      if (G_UNLIKELY (ret < 0)) {
-        GST_WARNING_OBJECT (tiger,
-            "Tiger renderer failed to set buffer to video frame: %d", ret);
-      } else {
-        kate_float t = gst_kate_tiger_get_time (tiger);
-        GST_LOG_OBJECT (tiger, "Video segment calc: last stop %ld, time %.3f",
-            (long) tiger->video_segment.last_stop, t);
-
-        ret = tiger_renderer_update (tiger->tr, t, 1);
-        if (G_UNLIKELY (ret < 0)) {
-          GST_WARNING_OBJECT (tiger, "Tiger renderer failed to update: %d",
-              ret);
-        } else {
-          ret = tiger_renderer_render (tiger->tr);
-          if (G_UNLIKELY (ret < 0)) {
-            GST_WARNING_OBJECT (tiger,
-                "Tiger renderer failed to render to video frame: %d", ret);
-          } else {
-            GST_LOG_OBJECT (tiger,
-                "Tiger renderer rendered on video frame at %f", t);
-          }
-        }
-      }
-    }
+    goto pass;
   }
 
+  /* and setup that buffer before rendering */
+  if (gst_video_format_is_yuv (tiger->video_format)) {
+    guint8 *tmp = g_realloc (tiger->render_buffer,
+        tiger->video_width * tiger->video_height * 4);
+    if (!tmp) {
+      GST_WARNING_OBJECT (tiger, "Failed to allocate render buffer");
+      goto pass;
+    }
+    tiger->render_buffer = tmp;
+    ptr = tiger->render_buffer;
+    tiger_renderer_set_surface_clear_color (tiger->tr, 1, 0.0, 0.0, 0.0, 0.0);
+  } else {
+    ptr = GST_BUFFER_DATA (buf);
+  }
+  ret =
+      tiger_renderer_set_buffer (tiger->tr, ptr, tiger->video_width,
+      tiger->video_height, tiger->video_width * 4, tiger->swap_rgb);
+  if (G_UNLIKELY (ret < 0)) {
+    GST_WARNING_OBJECT (tiger,
+        "Tiger renderer failed to set buffer to video frame: %d", ret);
+    goto pass;
+  }
+  ret = tiger_renderer_render (tiger->tr);
+  if (G_UNLIKELY (ret < 0)) {
+    GST_WARNING_OBJECT (tiger,
+        "Tiger renderer failed to render to video frame: %d", ret);
+  } else {
+    GST_LOG_OBJECT (tiger, "Tiger renderer rendered on video frame at %f", t);
+  }
+
+  if (gst_video_format_is_yuv (tiger->video_format)) {
+    gst_kate_tiger_blend_yuv (tiger, buf, tiger->render_buffer,
+        tiger->video_width, tiger->video_height);
+  }
+
+pass:
   GST_KATE_TIGER_MUTEX_UNLOCK (tiger);
 
   rflow = gst_pad_push (tiger->srcpad, buf);
@@ -755,6 +1296,8 @@ gst_kate_tiger_change_state (GstElement * element, GstStateChange transition)
     case GST_STATE_CHANGE_PAUSED_TO_READY:
       GST_DEBUG_OBJECT (tiger, "PAUSED -> READY, clearing kate state");
       GST_KATE_TIGER_MUTEX_LOCK (tiger);
+      gst_kate_util_decoder_base_set_flushing (&tiger->decoder, TRUE);
+      g_cond_broadcast (tiger->cond);
       if (tiger->tr) {
         tiger_renderer_destroy (tiger->tr);
         tiger->tr = NULL;
@@ -818,9 +1361,13 @@ gst_kate_tiger_seek (GstKateTiger * tiger, GstPad * pad, GstEvent * event)
   gst_event_parse_seek (event, &rate, &format, &flags, &cur_type, &cur,
       &stop_type, &stop);
 
+  if (flags & GST_SEEK_FLAG_FLUSH)
+    gst_pad_push_event (tiger->srcpad, gst_event_new_flush_start ());
+
   GST_KATE_TIGER_MUTEX_LOCK (tiger);
   tiger->video_flushing = TRUE;
   gst_kate_util_decoder_base_set_flushing (&tiger->decoder, TRUE);
+  g_cond_broadcast (tiger->cond);
   GST_KATE_TIGER_MUTEX_UNLOCK (tiger);
 
   if (format == GST_FORMAT_TIME) {
@@ -904,6 +1451,7 @@ gst_kate_tiger_handle_kate_event (GstPad * pad, GstEvent * event)
     case GST_EVENT_NEWSEGMENT:
       GST_INFO_OBJECT (tiger, "New segment on Kate pad");
       GST_KATE_TIGER_MUTEX_LOCK (tiger);
+      g_cond_broadcast (tiger->cond);
       gst_kate_util_decoder_base_new_segment_event (&tiger->decoder, event);
       GST_KATE_TIGER_MUTEX_UNLOCK (tiger);
       gst_event_unref (event);
@@ -912,6 +1460,7 @@ gst_kate_tiger_handle_kate_event (GstPad * pad, GstEvent * event)
       GST_KATE_TIGER_MUTEX_LOCK (tiger);
       gst_kate_util_decoder_base_set_flushing (&tiger->decoder, TRUE);
       GST_KATE_TIGER_MUTEX_UNLOCK (tiger);
+      g_cond_broadcast (tiger->cond);
       gst_event_unref (event);
       break;
     case GST_EVENT_FLUSH_STOP:
@@ -924,6 +1473,9 @@ gst_kate_tiger_handle_kate_event (GstPad * pad, GstEvent * event)
       /* we ignore this, it just means we don't have anymore Kate packets, but
          the Tiger renderer will still draw (if appropriate) on incoming video */
       GST_INFO_OBJECT (tiger, "EOS on Kate pad");
+      GST_KATE_TIGER_MUTEX_LOCK (tiger);
+      g_cond_broadcast (tiger->cond);
+      GST_KATE_TIGER_MUTEX_UNLOCK (tiger);
       gst_event_unref (event);
       break;
     default:
@@ -1000,6 +1552,7 @@ gst_kate_tiger_handle_video_event (GstPad * pad, GstEvent * event)
       gst_segment_init (&tiger->video_segment, GST_FORMAT_UNDEFINED);
       tiger->video_flushing = TRUE;
       GST_KATE_TIGER_MUTEX_UNLOCK (tiger);
+      g_cond_broadcast (tiger->cond);
       res = gst_pad_event_default (pad, event);
       break;
     case GST_EVENT_FLUSH_STOP:
