@@ -88,6 +88,8 @@
 #include <stdio.h>
 #include <string.h>
 
+#include <gst/video/video.h>
+
 #include "mpegtsmux.h"
 
 #include "mpegtsmux_h264.h"
@@ -151,6 +153,8 @@ static void mpegtsmux_release_pad (GstElement * element, GstPad * pad);
 static GstStateChangeReturn mpegtsmux_change_state (GstElement * element,
     GstStateChange transition);
 static void mpegtsdemux_set_header_on_caps (MpegTsMux * mux);
+static gboolean mpegtsmux_sink_event (GstPad * pad, GstEvent * event);
+static gboolean mpegtsmux_src_event (GstPad * pad, GstEvent * event);
 
 GST_BOILERPLATE (MpegTsMux, mpegtsmux, GstElement, GST_TYPE_ELEMENT);
 
@@ -160,10 +164,10 @@ mpegtsmux_base_init (gpointer g_class)
 
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&mpegtsmux_sink_factory));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&mpegtsmux_src_factory));
+  gst_element_class_add_static_pad_template (element_class,
+      &mpegtsmux_sink_factory);
+  gst_element_class_add_static_pad_template (element_class,
+      &mpegtsmux_src_factory);
 
   gst_element_class_set_details_simple (element_class,
       "MPEG Transport Stream Muxer", "Codec/Muxer",
@@ -215,6 +219,7 @@ mpegtsmux_init (MpegTsMux * mux, MpegTsMuxClass * g_class)
   mux->srcpad =
       gst_pad_new_from_static_template (&mpegtsmux_src_factory, "src");
   gst_pad_use_fixed_caps (mux->srcpad);
+  gst_pad_set_event_function (mux->srcpad, mpegtsmux_src_event);
   gst_element_add_pad (GST_ELEMENT (mux), mux->srcpad);
 
   mux->collect = gst_collect_pads_new ();
@@ -238,6 +243,8 @@ mpegtsmux_init (MpegTsMux * mux, MpegTsMuxClass * g_class)
   mux->prog_map = NULL;
   mux->streamheader = NULL;
   mux->streamheader_sent = FALSE;
+  mux->force_key_unit_event = NULL;
+  mux->pending_key_unit_ts = GST_CLOCK_TIME_NONE;
 }
 
 static void
@@ -642,7 +649,9 @@ mpegtsmux_choose_best_stream (MpegTsMux * mux)
     }
   }
   if (c_best) {
-    gst_buffer_unref (gst_collect_pads_pop (mux->collect, c_best));
+    GstBuffer *buffer;
+    if ((buffer = gst_collect_pads_pop (mux->collect, c_best)))
+      gst_buffer_unref (buffer);
   }
 
   return best;
@@ -650,6 +659,201 @@ mpegtsmux_choose_best_stream (MpegTsMux * mux)
 
 #define COLLECT_DATA_PAD(collect_data) (((GstCollectData *)(collect_data))->pad)
 
+static MpegTsPadData *
+find_pad_data (MpegTsMux * mux, GstPad * pad)
+{
+  GSList *walk;
+  MpegTsPadData *ts_data = NULL;
+
+  GST_COLLECT_PADS_PAD_LOCK (mux->collect);
+  walk = mux->collect->abidata.ABI.pad_list;
+  while (walk) {
+    if (((GstCollectData *) walk->data)->pad == pad) {
+      ts_data = (MpegTsPadData *) walk->data;
+      break;
+    }
+
+    walk = g_slist_next (walk);
+  }
+  GST_COLLECT_PADS_PAD_UNLOCK (mux->collect);
+
+  return ts_data;
+}
+
+static gboolean
+mpegtsmux_sink_event (GstPad * pad, GstEvent * event)
+{
+  MpegTsMux *mux = GST_MPEG_TSMUX (gst_pad_get_parent (pad));
+  MpegTsPadData *ts_data;
+  gboolean res = TRUE;
+  gboolean forward = TRUE;
+
+  ts_data = find_pad_data (mux, pad);
+
+  switch (GST_EVENT_TYPE (event)) {
+    case GST_EVENT_CUSTOM_DOWNSTREAM:
+    {
+      GstClockTime timestamp, stream_time, running_time;
+      gboolean all_headers;
+      guint count;
+
+      if (!gst_video_event_is_force_key_unit (event))
+        goto out;
+
+      forward = FALSE;
+
+      gst_video_event_parse_downstream_force_key_unit (event,
+          &timestamp, &stream_time, &running_time, &all_headers, &count);
+      GST_INFO_OBJECT (mux, "have downstream force-key-unit event on pad %s, "
+          "seqnum %d, running-time %" GST_TIME_FORMAT " count %d",
+          gst_pad_get_name (pad), gst_event_get_seqnum (event),
+          GST_TIME_ARGS (running_time), count);
+
+      if (mux->force_key_unit_event != NULL) {
+        GST_INFO_OBJECT (mux, "skipping downstream force key unit event "
+            "as an upstream force key unit is already queued");
+        goto out;
+      }
+
+      if (!all_headers)
+        goto out;
+
+      mux->pending_key_unit_ts = running_time;
+      gst_event_replace (&mux->force_key_unit_event, event);
+      break;
+    }
+    default:
+      break;
+  }
+
+out:
+  if (forward)
+    res = ts_data->eventfunc (pad, event);
+
+  gst_object_unref (mux);
+  return res;
+}
+
+static gboolean
+mpegtsmux_src_event (GstPad * pad, GstEvent * event)
+{
+  MpegTsMux *mux = GST_MPEG_TSMUX (gst_pad_get_parent (pad));
+  gboolean res = TRUE;
+
+  switch (GST_EVENT_TYPE (event)) {
+    case GST_EVENT_CUSTOM_UPSTREAM:
+    {
+      GstIterator *iter;
+      GstIteratorResult iter_ret;
+      GstPad *sinkpad;
+      GstClockTime running_time;
+      gboolean all_headers, done;
+      guint count;
+
+      if (!gst_video_event_is_force_key_unit (event))
+        break;
+
+      gst_video_event_parse_upstream_force_key_unit (event,
+          &running_time, &all_headers, &count);
+
+      GST_INFO_OBJECT (mux, "received upstream force-key-unit event, "
+          "seqnum %d running_time %" GST_TIME_FORMAT " all_headers %d count %d",
+          gst_event_get_seqnum (event), GST_TIME_ARGS (running_time),
+          all_headers, count);
+
+      if (!all_headers)
+        break;
+
+      mux->pending_key_unit_ts = running_time;
+      gst_event_replace (&mux->force_key_unit_event, event);
+
+      iter = gst_element_iterate_sink_pads (GST_ELEMENT_CAST (mux));
+      done = FALSE;
+      while (!done) {
+        gboolean res = FALSE, tmp;
+        iter_ret = gst_iterator_next (iter, (gpointer *) & sinkpad);
+
+        switch (iter_ret) {
+          case GST_ITERATOR_DONE:
+            done = TRUE;
+            break;
+          case GST_ITERATOR_OK:
+            GST_INFO_OBJECT (mux, "forwarding to %s",
+                gst_pad_get_name (sinkpad));
+            tmp = gst_pad_push_event (sinkpad, gst_event_ref (event));
+            GST_INFO_OBJECT (mux, "result %d", tmp);
+            /* succeed if at least one pad succeeds */
+            res |= tmp;
+            gst_object_unref (sinkpad);
+            break;
+          case GST_ITERATOR_ERROR:
+            done = TRUE;
+            break;
+          case GST_ITERATOR_RESYNC:
+            break;
+        }
+      }
+
+      gst_event_unref (event);
+      break;
+    }
+    default:
+      res = gst_pad_event_default (pad, event);
+      break;
+  }
+
+  gst_object_unref (mux);
+  return res;
+}
+
+static GstEvent *
+check_pending_key_unit_event (GstEvent * pending_event, GstSegment * segment,
+    GstClockTime timestamp, guint flags, GstClockTime pending_key_unit_ts)
+{
+  GstClockTime running_time, stream_time;
+  gboolean all_headers;
+  guint count;
+  GstEvent *event = NULL;
+
+  g_return_val_if_fail (pending_event != NULL, NULL);
+  g_return_val_if_fail (segment != NULL, NULL);
+
+  if (pending_event == NULL)
+    goto out;
+
+  if (GST_CLOCK_TIME_IS_VALID (pending_key_unit_ts) &&
+      timestamp == GST_CLOCK_TIME_NONE)
+    goto out;
+
+  running_time = gst_segment_to_running_time (segment,
+      GST_FORMAT_TIME, timestamp);
+
+  GST_INFO ("now %" GST_TIME_FORMAT " wanted %" GST_TIME_FORMAT,
+      GST_TIME_ARGS (running_time), GST_TIME_ARGS (pending_key_unit_ts));
+  if (GST_CLOCK_TIME_IS_VALID (pending_key_unit_ts) &&
+      running_time < pending_key_unit_ts)
+    goto out;
+
+  if (flags & GST_BUFFER_FLAG_DELTA_UNIT) {
+    GST_INFO ("pending force key unit, waiting for keyframe");
+    goto out;
+  }
+
+  stream_time = gst_segment_to_stream_time (segment,
+      GST_FORMAT_TIME, timestamp);
+
+  gst_video_event_parse_upstream_force_key_unit (pending_event,
+      NULL, &all_headers, &count);
+
+  event =
+      gst_video_event_new_downstream_force_key_unit (timestamp, stream_time,
+      running_time, all_headers, count);
+  gst_event_set_seqnum (event, gst_event_get_seqnum (pending_event));
+
+out:
+  return event;
+}
+
 static GstFlowReturn
 mpegtsmux_collected (GstCollectPads * pads, MpegTsMux * mux)
 {
@@ -679,22 +883,56 @@ mpegtsmux_collected (GstCollectPads * pads, MpegTsMux * mux)
     if (prog == NULL) {
       GST_ELEMENT_ERROR (mux, STREAM, MUX,
           ("Stream on pad %" GST_PTR_FORMAT
-              " is not associated with any program", best), (NULL));
+              " is not associated with any program", COLLECT_DATA_PAD (best)),
+          (NULL));
       return GST_FLOW_ERROR;
     }
 
-    if (G_UNLIKELY (prog->pcr_stream == NULL)) {
-      if (best) {
-        /* Take the first data stream for the PCR */
-        GST_DEBUG_OBJECT (COLLECT_DATA_PAD (best),
-            "Use stream (pid=%d) from pad as PCR for program (prog_id = %d)",
-            MPEG_TS_PAD_DATA (best)->pid, MPEG_TS_PAD_DATA (best)->prog_id);
+    if (mux->force_key_unit_event != NULL) {
+      GstEvent *event;
 
-        /* Set the chosen PCR stream */
-        tsmux_program_set_pcr_stream (prog, best->stream);
+      event = check_pending_key_unit_event (mux->force_key_unit_event,
+          &best->collect.segment, GST_BUFFER_TIMESTAMP (buf),
+          GST_BUFFER_FLAGS (buf), mux->pending_key_unit_ts);
+      if (event) {
+        GstClockTime running_time;
+        guint count;
+        GList *cur;
+
+        mux->pending_key_unit_ts = GST_CLOCK_TIME_NONE;
+        gst_event_replace (&mux->force_key_unit_event, NULL);
+
+        gst_video_event_parse_downstream_force_key_unit (event,
+            NULL, NULL, &running_time, NULL, &count);
+
+        GST_INFO_OBJECT (mux, "pushing downstream force-key-unit event %d "
+            "%" GST_TIME_FORMAT " count %d", gst_event_get_seqnum (event),
+            GST_TIME_ARGS (running_time), count);
+        gst_pad_push_event (mux->srcpad, event);
+
+        /* output PAT */
+        mux->tsmux->last_pat_ts = -1;
+
+        /* output PMT for each program */
+        for (cur = g_list_first (mux->tsmux->programs); cur != NULL;
+            cur = g_list_next (cur)) {
+          TsMuxProgram *program = (TsMuxProgram *) cur->data;
+
+          program->last_pmt_ts = -1;
+        }
       }
     }
 
+    if (G_UNLIKELY (prog->pcr_stream == NULL)) {
+      /* Take the first data stream for the PCR */
+      GST_DEBUG_OBJECT (COLLECT_DATA_PAD (best),
+          "Use stream (pid=%d) from pad as PCR for program (prog_id = %d)",
+          MPEG_TS_PAD_DATA (best)->pid, MPEG_TS_PAD_DATA (best)->prog_id);
+
+      /* Set the chosen PCR stream */
+      tsmux_program_set_pcr_stream (prog, best->stream);
+    }
+
     g_return_val_if_fail (buf != NULL, GST_FLOW_ERROR);
     if (best->stream->is_video_stream)
       delta = GST_BUFFER_FLAG_IS_SET (buf, GST_BUFFER_FLAG_DELTA_UNIT);
@@ -764,6 +1002,9 @@ mpegtsmux_request_new_pad (GstElement * element,
   if (pad_data == NULL)
     goto pad_failure;
 
+  pad_data->eventfunc = pad->eventfunc;
+  gst_pad_set_event_function (pad, mpegtsmux_sink_event);
+
   pad_data->pid = pid;
   pad_data->last_ts = GST_CLOCK_TIME_NONE;
   pad_data->codec_data = NULL;
